@@ -53,9 +53,19 @@ namespace studio {
 
 class AsyncRenderer : public etl::shared_object, public sigc::trackable
 {
-	//! Signal emmited when target has been stopped or has finished
-	sigc::signal<void> signal_finished_;
-	//! Signal emmited when target has succedded
+public:
+	enum Status {
+		RENDERING_UNDEFINED,
+		RENDERING_ERROR,
+		RENDERING_SUCCESS
+	};
+
+private:
+	//! Signal emitted when target has been stopped or has finished
+	//! An error message is passed as argument in case of error;
+	//! it's empty otherwise
+	sigc::signal<void, std::string> signal_finished_;
+	//! Signal emitted when target has succeeded
 	sigc::signal<void> signal_success_;
 
 	//! Seems to be unused
@@ -64,13 +74,12 @@ class AsyncRenderer : public etl::shared_object, public sigc::trackable
 	//! The target that is going to be asynchronously rendered.
 	etl::handle<synfig::Target> target;
 
-	//! Set to true when target render fails.
-	bool error;
-	//! Set to true when target render succedded
-	bool success;
+	//! Current rendering status
+	//! Undefined if not finished
+	Status status;
 
 	synfig::ProgressCallback *cb;
-	//! Signal to be emmited when the target is requested to stop
+	//! Signal to be emitted when the target is requested to stop
 	sigc::signal<void> signal_stop_;
 
 	Glib::Thread* render_thread;
@@ -95,17 +104,12 @@ public:
 	void stop();
 	void pause();
 	void resume();
-#ifdef SINGLE_THREADED
-	void rendering_progress();
-	bool updating;
-#endif
 
-	bool has_error()const { return error; }
-	bool has_success()const { return success; }
+	Status get_status() const { return status; }
 	synfig::Real get_execution_time() const { return (finish_time - start_time).as_double(); }
 	synfig::Real get_execution_clock() const { return (synfig::Real)(finish_clock - start_clock)/(synfig::Real)CLOCKS_PER_SEC; }
 
-	sigc::signal<void>& signal_finished() { return signal_finished_; }
+	sigc::signal<void, std::string>& signal_finished() { return signal_finished_; }
 	sigc::signal<void>& signal_success() { return signal_success_; }
 
 private:
