@@ -31,23 +31,29 @@
 #	include <config.h>
 #endif
 
-#include <synfig/general.h>
-
-#include "docks/dock_timetrack.h"
-#include "app.h"
+#include <cassert>
 
 #include <gtkmm/scrolledwindow.h>
-#include <cassert>
-#include "instance.h"
-#include "canvasview.h"
-#include "trees/layerparamtreestore.h"
-#include "workarea.h"
-#include "widgets/widget_timeslider.h"
-#include "widgets/widget_keyframe_list.h"
-#include <gui/localization.h>
-#include "trees/layertree.h"
+
+#include <synfig/general.h>
 #include <synfig/timepointcollect.h>
 
+#include <gui/helpers.h>
+#include <app.h>
+#include <instance.h>
+#include <canvasview.h>
+#include <workarea.h>
+#include <trees/layertree.h>
+#include <trees/layerparamtreestore.h>
+#include <widgets/widget_canvastimeslider.h>
+#include <widgets/widget_keyframe_list.h>
+
+#include "dock_timetrack.h"
+
+#include <gui/localization.h>
+
+#include <gui/exception_guard.h>
+
 #endif
 
 /* === U S I N G =========================================================== */
@@ -76,7 +82,8 @@ public:
 
 	void set_canvas_view(handle<CanvasView> canvas_view)
 	{
-		cellrenderer_time_track->set_adjustment(canvas_view->time_adjustment());
+		cellrenderer_time_track->set_time_model(canvas_view->time_model());
+		cellrenderer_time_track->set_canvas_interface(canvas_view->canvas_interface());
 	}
 
 	TimeTrackView()
@@ -116,7 +123,7 @@ public:
 			column2->pack_end(*text_cellrenderer,false);
 			text_cellrenderer->set_fixed_size (1,-1);
 
-			// Add a fixed size (1pixel widht, same height than value_type icon) empty (alpha) icon
+			// Add a fixed size (1pixel width, same height than value_type icon) empty (alpha) icon
 			// to align the rows with params dock when the text is smaller than value_type icons height
 			Gtk::CellRendererPixbuf* icon_cellrenderer = Gtk::manage( new Gtk::CellRendererPixbuf() );
 			Glib::RefPtr<Gdk::Pixbuf> pixbuf;
@@ -131,7 +138,7 @@ public:
 			column->set_resizable();
 			column->set_expand(true);
 // Commented during Align rows fixing
-// http://www.synfig.org/issues/thebuggenie/synfig/issues/161
+// https://www.synfig.org/issues/thebuggenie/synfig/issues/161
 // 			column->set_min_width(200);
 			
 			column2->set_resizable();
@@ -151,34 +158,28 @@ public:
 	bool
 	on_event(GdkEvent *event)
 	{
+		SYNFIG_EXCEPTION_GUARD_BEGIN()
 		switch(event->type)
 		{
+		case GDK_2BUTTON_PRESS:
 		case GDK_SCROLL:
-			if(mimic_tree_view)
-			{
+			if (mimic_tree_view) {
 				if(event->scroll.direction==GDK_SCROLL_DOWN)
-				{
-					mimic_tree_view->get_vadjustment()->set_value(
-						std::min(
-							mimic_tree_view->get_vadjustment()->get_value()+
-							mimic_tree_view->get_vadjustment()->get_step_increment(),
-							mimic_tree_view->get_vadjustment()->get_upper()-
-							mimic_tree_view->get_vadjustment()->get_page_size()
-						)
-					);
-					mimic_tree_view->get_vadjustment()->value_changed();
-				}
-				else if(event->scroll.direction==GDK_SCROLL_UP)
-				{
-					mimic_tree_view->get_vadjustment()->set_value(
-						std::max(
-							mimic_tree_view->get_vadjustment()->get_value()-
-							mimic_tree_view->get_vadjustment()->get_step_increment(),
-							mimic_tree_view->get_vadjustment()->get_lower()
-						)
-					);
-					mimic_tree_view->get_vadjustment()->value_changed();
-				}
+					ConfigureAdjustment(mimic_tree_view->get_vadjustment())
+						.set_value( std::min(
+							mimic_tree_view->get_vadjustment()->get_value()
+						  +	mimic_tree_view->get_vadjustment()->get_step_increment(),
+							mimic_tree_view->get_vadjustment()->get_upper()
+						  - mimic_tree_view->get_vadjustment()->get_page_size() ))
+						.finish();
+				else
+				if(event->scroll.direction==GDK_SCROLL_UP)
+					ConfigureAdjustment(mimic_tree_view->get_vadjustment())
+						.set_value( std::max(
+							mimic_tree_view->get_vadjustment()->get_value()
+						  -	mimic_tree_view->get_vadjustment()->get_step_increment(),
+							mimic_tree_view->get_vadjustment()->get_lower() ))
+						.finish();
 			}
 			break;
 		case GDK_BUTTON_PRESS:
@@ -292,6 +293,7 @@ public:
 		}
 		mimic_resync();
 		return Gtk::TreeView::on_event(event);
+		SYNFIG_EXCEPTION_GUARD_END_BOOL(true)
 	}
 
 	void
@@ -303,9 +305,9 @@ public:
 	void set_model(Glib::RefPtr<LayerParamTreeStore> store)
 	{
 		Gtk::TreeView::set_model(store);
-		param_tree_store_=store;
-		cellrenderer_time_track->set_canvas_interface(param_tree_store_->canvas_interface());
 		store->signal_changed().connect(sigc::mem_fun(*this, &TimeTrackView::queue_draw));
+		cellrenderer_time_track->set_canvas_interface( store->canvas_interface() );
+		param_tree_store_ = store;
 	}
 
 	void
@@ -348,6 +350,15 @@ public:
 					)
 			))
 		);
+		param_tree_view->get_selection()->signal_changed().connect(
+			sigc::bind(
+				sigc::mem_fun(
+					*this,
+					&TimeTrackView::on_external_selection_changed
+				),
+				param_tree_view->get_selection()
+			)
+		);
 		mimic_resync();
 	}
 
@@ -358,8 +369,10 @@ public:
 			Glib::RefPtr<Gtk::Adjustment> adjustment(mimic_tree_view->get_vadjustment());
 			set_vadjustment(adjustment);
 
-			if(adjustment->get_page_size()>get_height())
-				adjustment->set_page_size(get_height());
+			if (adjustment->get_page_size()>get_height())
+				ConfigureAdjustment(adjustment)
+					.set_page_size(get_height())
+					.finish();
 /* Commented during Align rows fixing
 // http://www.synfig.org/issues/thebuggenie/synfig/issues/161
 			int row_height = 0;
@@ -398,6 +411,14 @@ public:
 		if (!waypoint_set.empty())
 			signal_waypoint_clicked_timetrackview(value_desc,waypoint_set,button);
 	}
+	
+	void
+	on_external_selection_changed(Glib::RefPtr<TreeView::Selection> tree_view_selection)
+	{
+		Glib::RefPtr<TreeView::Selection> selection = get_selection();
+		for (const auto& path : tree_view_selection->get_selected_rows())
+			selection->select(path);
+	}
 };
 
 /* === G L O B A L S ======================================================= */
@@ -407,25 +428,16 @@ public:
 /* === M E T H O D S ======================================================= */
 
 Dock_Timetrack::Dock_Timetrack():
-	Dock_CanvasSpecific("timetrack",_("Timetrack"),Gtk::StockID("synfig-timetrack"))
+	Dock_CanvasSpecific("timetrack",_("Timetrack"),Gtk::StockID("synfig-timetrack")),
+	grid_(),
+	mimic_tree_view()
 {
-	table_=0;
-	widget_timeslider_= new Widget_Timeslider();
-	widget_kf_list_= new Widget_Keyframe_List();
-	
 	set_use_scrolled(false);
-
-	hscrollbar_=new Gtk::HScrollbar();
-	vscrollbar_=new Gtk::VScrollbar();
 }
 
 Dock_Timetrack::~Dock_Timetrack()
 {
-	if(table_)delete table_;
-	delete hscrollbar_;
-	delete vscrollbar_;
-	delete widget_timeslider_;
-	delete widget_kf_list_;
+	if (grid_) delete grid_;
 }
 
 void
@@ -450,17 +462,9 @@ Dock_Timetrack::init_canvas_view_vfunc(etl::loose_handle<CanvasView> canvas_view
 
 	studio::LayerTree* tree_layer(dynamic_cast<studio::LayerTree*>(canvas_view->get_ext_widget("layers_cmp")));
 
-	/*
-	if(!getenv("SYNFIG_TIMETRACK_HEADER_HEIGHT"))
-	*/
 	tree_layer->signal_param_tree_header_height_changed().connect(sigc::mem_fun(*this, &studio::Dock_Timetrack::on_update_header_height));
-
-	canvas_view->time_adjustment()->signal_value_changed().connect(sigc::mem_fun(*tree_view,&Gtk::TreeView::queue_draw));
-	canvas_view->time_adjustment()->signal_changed().connect(sigc::mem_fun(*tree_view,&Gtk::TreeView::queue_draw));
-
+	canvas_view->time_model()->signal_changed().connect(sigc::mem_fun(*tree_view,&Gtk::TreeView::queue_draw));
 	canvas_view->set_ext_widget(get_name(),tree_view);
-	// widget_timeslider fps connection to animation render description change
-	canvas_view->canvas_interface()->signal_rend_desc_changed().connect(sigc::mem_fun(*this,&studio::Dock_Timetrack::refresh_rend_desc));
 }
 
 void
@@ -485,116 +489,113 @@ Dock_Timetrack::refresh_selected_param()
 */
 }
 
-/*! \fn Dock_Timetrack::refresh_rend_desc()
-**	\brief Signal handler for animation render description change
-*/
-void
-Dock_Timetrack::refresh_rend_desc()
-{
-	if(App::get_selected_canvas_view())
-	{
-		widget_timeslider_->set_global_fps(App::get_selected_canvas_view()->get_canvas()->rend_desc().get_frame_rate());
-	}
-}
-
 void
 Dock_Timetrack::changed_canvas_view_vfunc(etl::loose_handle<CanvasView> canvas_view)
 {
-	if(table_)
+	if(grid_)
 	{
-		table_->hide();
-		remove(*table_);
-		clear_previous();
-		delete table_;
-		hscrollbar_->unset_adjustment();
-		vscrollbar_->unset_adjustment();
-		//widget_timeslider_->unset_adjustment();
-		table_=0;
+		hscrollbar_.unset_adjustment();
+		vscrollbar_.unset_adjustment();
+
+		widget_timeslider_.set_canvas_view( CanvasView::Handle() );
+
+		widget_kf_list_.set_time_model( etl::handle<TimeModel>() );
+		widget_kf_list_.set_canvas_interface( etl::loose_handle<synfigapp::CanvasInterface>() );
+
+		delete grid_;
+		grid_=0;
 	}
 
 	if(canvas_view)
 	{
 		TimeTrackView* tree_view(dynamic_cast<TimeTrackView*>(canvas_view->get_ext_widget(get_name())));
 		Gtk::TreeView* param_tree_view(dynamic_cast<Gtk::TreeView*>(canvas_view->get_ext_widget("params")));
+		assert(tree_view);
+		assert(param_tree_view);
+
 		Gtk::ScrolledWindow* scrolled = Gtk::manage(new Gtk::ScrolledWindow);
+		scrolled->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_ALWAYS);
 		scrolled->get_vscrollbar()->hide();
 		scrolled->add(*tree_view);
-		scrolled->set_policy(Gtk::POLICY_NEVER,Gtk::POLICY_AUTOMATIC);
 		scrolled->set_vadjustment(param_tree_view->get_vadjustment());
 		scrolled->show_all();
 
-		assert(tree_view);
 		// Fixed size drawing areas to align the widget_timeslider and tree_view time cursors
 		// TODO ?: one align_drawingArea.(0, 1, 0, 1) modify_bg KF's color another (0, 1, 1, 2) modify_bg TS's color
 		Gtk::DrawingArea* align_drawingArea1 = Gtk::manage(new Gtk::DrawingArea);
 		// TODO ?: one align_drawingArea.(2, 3, 0, 1) modify_bg KF's color another (2, 3, 1, 2) modify_bg TS's color
 		Gtk::DrawingArea* align_drawingArea2 = Gtk::manage(new Gtk::DrawingArea);
-#if (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION >= 14)
-		align_drawingArea1->set_size_request(2,-1);
-		align_drawingArea2->set_size_request(4,-1);
-#else
+#if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION < 14)
 		align_drawingArea1->set_size_request(4,-1);
 		align_drawingArea2->set_size_request(9,-1);
+#else
+		align_drawingArea1->set_size_request(2,-1);
+		align_drawingArea2->set_size_request(4,-1);
 #endif
-		widget_timeslider_->set_time_adjustment(canvas_view->time_adjustment());
-		widget_timeslider_->set_bounds_adjustment(canvas_view->time_window_adjustment());
-		widget_timeslider_->set_global_fps(canvas_view->get_canvas()->rend_desc().get_frame_rate());
 
-		widget_kf_list_->set_time_adjustment(canvas_view->time_adjustment());
-		widget_kf_list_->set_canvas_interface(canvas_view->canvas_interface());
+		widget_timeslider_.set_canvas_view(canvas_view);
 
-		vscrollbar_->set_adjustment(tree_view->get_vadjustment());
-		hscrollbar_->set_adjustment(canvas_view->time_window_adjustment());
+		widget_kf_list_.set_time_model(canvas_view->time_model());
+		widget_kf_list_.set_canvas_interface(canvas_view->canvas_interface());
 
-/*
-	0------1------2------3------4
-	|  A   |  KF  |  A   |  v   |
-	|  L   |      |  L   |  s   |
-	1--I---x------x--I---x--c---x
-	|  G   |  TS  |  G   |  r   |
-	|  N1  |      |  N2  |  o   |
-	2------x------x------x--l---x
-	|  TV  |  TV  |  TV  |  l   |
-	|      |      |      |  b   |
-	3------x------x------x------x
-	| hscrollbar
+		vscrollbar_.set_adjustment(scrolled->get_vadjustment());
+		hscrollbar_.set_adjustment(canvas_view->time_model()->scroll_time_adjustment());
 
-KF = widget_kf_list
-TS = widget_timeslider
-TV = tree_view
-ALIGN1 = align_drawingArea1
-ALIGN2 = align_drawingArea2
-*/
-		table_=new Gtk::Table(3,4);
-		table_->attach(*align_drawingArea1, 0, 1, 0, 2, Gtk::SHRINK, Gtk::FILL);
-		table_->attach(*widget_kf_list_, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::SHRINK);
-		table_->attach(*widget_timeslider_, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::SHRINK);
-		table_->attach(*align_drawingArea2, 2, 3, 0, 2, Gtk::SHRINK, Gtk::FILL);
-		table_->attach(*scrolled, 0, 3, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
-		table_->attach(*hscrollbar_, 0, 3, 3, 4, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::SHRINK);
-		table_->attach(*vscrollbar_, 3, 4, 0, 3, Gtk::FILL|Gtk::SHRINK, Gtk::FILL|Gtk::EXPAND);
-		add(*table_);
+		//  0------1------2------3------4
+		//  |  A   |  KF  |  A   |  v   |
+		//  |  L   |      |  L   |  s   |
+		//  1  I   x------x  I   x  c   x
+		//  |  G   |  TS  |  G   |  r   |
+		//  |  N1  |      |  N2  |  o   |
+		//  2------x------x------x  l   x
+		//  |  TV     TV     TV  |  l   |
+		//  |                    |  b   |
+		//  3------x------x------x------x
+		//  | hscrollbar                |
+		//  |                           |
+		//  4------x------x------x------x
+		//
+		// KF = widget_kf_list
+		// TS = widget_timeslider
+		// TV = tree_view
+		// ALIGN1 = align_drawingArea1
+		// ALIGN2 = align_drawingArea2
+
+		widget_kf_list_.set_hexpand();
+		widget_timeslider_.set_hexpand();
+		scrolled->set_hexpand();
+		scrolled->set_vexpand();
+		vscrollbar_.set_vexpand();
+		hscrollbar_.set_hexpand();
+		
+		grid_=new Gtk::Grid();
+		grid_->set_column_homogeneous(false);
+		grid_->set_row_homogeneous(false);
+		grid_->attach(*align_drawingArea1, 0, 0, 1, 2);
+		grid_->attach(widget_kf_list_,     1, 0, 1, 1);
+		grid_->attach(widget_timeslider_,  1, 1, 1, 1);
+		grid_->attach(*align_drawingArea2, 2, 0, 1, 2);
+		grid_->attach(*scrolled,           0, 2, 3, 1);
+		grid_->attach(vscrollbar_,         3, 0, 1, 3);
+		grid_->attach(hscrollbar_,         0, 3, 4, 1);
+		add(*grid_);
 		
 		// Should be here, after the widget was attached to table
 		tree_view->add_events(Gdk::SCROLL_MASK);
 
 		//add(*last_widget_curves_);
-		table_->show_all();
+		grid_->show_all();
 		show_all();
 	}
-	else
-	{
-		//clear_previous();
-	}
 }
 
 void
-Dock_Timetrack::on_update_header_height( int header_height)
+Dock_Timetrack::on_update_header_height( int /*header_height*/)
 {
 	int width=0;
 	int height=0;
 	int kf_list_height=10;
 	mimic_tree_view->convert_bin_window_to_widget_coords(0, 0, width, height);
-	widget_timeslider_->set_size_request(-1,height-kf_list_height);
-	widget_kf_list_->set_size_request(-1,kf_list_height);
+	widget_timeslider_.set_size_request(-1,height-kf_list_height);
+	widget_kf_list_.set_size_request(-1,kf_list_height);
 }
