@@ -564,10 +564,10 @@ StateCircle_Context::StateCircle_Context(CanvasView* canvas_view):
 		("synfig-layer_geometry_region"), _("Create a region layer"));
 
 	LAYER_CREATION(layer_outline_togglebutton,
-		("synfig-layer_geometry_outline"), _("Create a outline layer"));
+		("synfig-layer_geometry_outline"), _("Create an outline layer"));
 
 	LAYER_CREATION(layer_advanced_outline_togglebutton,
-		("synfig-layer_geometry_advanced_outline"), _("Create a advanced outline layer"));
+		("synfig-layer_geometry_advanced_outline"), _("Create an advanced outline layer"));
 
 	LAYER_CREATION(layer_plant_togglebutton,
 		("synfig-layer_other_plant"), _("Create a plant layer"));
@@ -769,7 +769,8 @@ StateCircle_Context::StateCircle_Context(CanvasView* canvas_view):
 	options_table.set_row_spacings(GAP); // row gap
 	options_table.set_row_spacing(0, GAP*2); // the gap between first and second row.
 	options_table.set_row_spacing(2, 1); // row gap between label and icon of layer type
-	options_table.set_row_spacing(14, 0); // the final row using border width of table
+	//options_table.set_row_spacing(13, 0); // the final row using border width of table
+	options_table.set_margin_bottom(0);
 
 	options_table.show_all();
 
@@ -889,12 +890,13 @@ StateCircle_Context::make_circle(const Point& _p1, const Point& _p2)
 	std::vector<BLinePoint> new_list;
 	for (int i = 0; i < points; i++)
 	{
-		new_list.push_back(*(new BLinePoint));
-		new_list[i].set_width(1);
-		new_list[i].set_vertex(Point(radius*Angle::cos(angle*i + offset).get() + x,
+		BLinePoint p;
+		p.set_width(1);
+		p.set_vertex(Point(radius*Angle::cos(angle*i + offset).get() + x,
 									 radius*Angle::sin(angle*i + offset).get() + y));
-		new_list[i].set_tangent(Point(-radius*tangent*Angle::sin(angle*i + offset).get(),
+		p.set_tangent(Point(-radius*tangent*Angle::sin(angle*i + offset).get(),
 									   radius*tangent*Angle::cos(angle*i + offset).get()));
+		new_list.push_back(p);
 	}
 
 	ValueNode_BLine::Handle value_node_bline(ValueNode_BLine::create(new_list));
@@ -1524,7 +1526,7 @@ StateCircle_Context::toggle_layer_creation()
 
 	// feather size
 	if (get_layer_circle_flag() ||
-		get_layer_circle_flag() ||
+		//get_layer_circle_flag() ||
 		get_layer_region_flag() ||
 		get_layer_outline_flag() ||
 		get_layer_advanced_outline_flag())
