@@ -55,6 +55,8 @@
 
 #include <gui/localization.h>
 
+#include <gui/exception_guard.h>
+
 #endif
 
 /* === U S I N G =========================================================== */
@@ -204,6 +206,7 @@ StateMirror_Context::StateMirror_Context(CanvasView* canvas_view):
 bool
 StateMirror_Context::key_press_event(GdkEventKey *event)
 {
+	SYNFIG_EXCEPTION_GUARD_BEGIN()
 	if (event->keyval==GDK_KEY_Shift_L || event->keyval==GDK_KEY_Shift_R)
 	{
 		if (shift_is_pressed) return false;
@@ -213,11 +216,13 @@ StateMirror_Context::key_press_event(GdkEventKey *event)
 	}
 
 	return false; //Pass on the event to other handlers, just in case
+	SYNFIG_EXCEPTION_GUARD_END_BOOL(true)
 }
 
 bool
 StateMirror_Context::key_release_event(GdkEventKey *event)
 {
+	SYNFIG_EXCEPTION_GUARD_BEGIN()
 	if (event->keyval==GDK_KEY_Shift_L || event->keyval==GDK_KEY_Shift_R )
 	{
 		if (!shift_is_pressed) return false;
@@ -227,6 +232,7 @@ StateMirror_Context::key_release_event(GdkEventKey *event)
 	}
 
 	return false; //Pass on the event to other handlers, just in case
+	SYNFIG_EXCEPTION_GUARD_END_BOOL(true)
 }
 
 void
