@@ -30,23 +30,21 @@
 #	include <config.h>
 #endif
 
-#include <sigc++/signal.h>
-#include <sigc++/object.h>
+#include "app.h"
+#include <synfig/general.h>
 
 #include <ETL/handle>
 #include <synfig/vector.h>
 
-
 #include "state_zoom.h"
 #include "state_normal.h"
 #include "event_mouse.h"
 #include "canvasview.h"
 #include "workarea.h"
-#include "app.h"
 #include "docks/dock_toolbox.h"
 #include <synfigapp/main.h>
 
-#include "general.h"
+#include <gui/localization.h>
 
 #endif
 
@@ -117,7 +115,7 @@ StateZoom::~StateZoom()
 StateZoom_Context::StateZoom_Context(CanvasView* canvas_view):
 	canvas_view_(canvas_view),
 	is_working(*canvas_view),
-	push_state(get_work_area())
+	push_state(*get_work_area())
 {
 	// Turn off layer clicking
 	get_work_area()->set_allow_layer_clicks(false);
@@ -172,7 +170,7 @@ StateZoom_Context::event_mouse_click_handler(const Smach::event& x)
 			if(event.modifier & Gdk::CONTROL_MASK) //zoom out...
 			{
 				if (window_span[0] == 0 || window_span[1] == 0) zoom = 1;
-				else zoom = max(abs(box_span[0]/window_span[0]), abs(box_span[1]/window_span[1]));
+				else zoom = std::max(std::fabs(box_span[0]/window_span[0]), std::fabs(box_span[1]/window_span[1]));
 
 				// focus_point is -1 times the real position for some reason...
 				// center the window so the old contents fill the drawn box
@@ -181,7 +179,7 @@ StateZoom_Context::event_mouse_click_handler(const Smach::event& x)
 			else				// zoom in
 			{
 				if (box_span[0] == 0 || box_span[1] == 0) zoom = 1;
-				else zoom = min(abs(window_span[0]/box_span[0]), abs(window_span[1]/box_span[1]));
+				else zoom = std::min(std::fabs(window_span[0]/box_span[0]), std::fabs(window_span[1]/box_span[1]));
 
 				// center the window at the center of the box
 				newpos = -(-get_work_area()->get_focus_point() + (box_middle - window_middle));
