@@ -68,6 +68,8 @@ public:
 public:
 
 	KeyframeTreeStore::Model model;
+	synfig::Keyframe selected_kf;
+
 
 	/*
  -- ** -- P R I V A T E   D A T A ---------------------------------------------
@@ -93,6 +95,8 @@ private:
 
 	bool editable_;
 
+	bool send_selection;
+
 	/*
  -- ** -- P R I V A T E   M E T H O D S ---------------------------------------
 	*/
@@ -113,10 +117,18 @@ private:
 
 	void on_edited_description(const Glib::ustring&path_string,const Glib::ustring &description);
 
-	bool on_event(GdkEvent *event);
+	virtual bool on_event(GdkEvent *event);
 
 	void on_rend_desc_changed();
 
+	//! Action performed when a keyframe is selected in the widget
+	//! This is where keyframe selected signal is fired
+	void on_selection_changed();
+
+	//! Signal handler for select keyframe signal from canvas interface
+	void on_keyframe_selected(synfig::Keyframe);
+	sigc::connection	keyframeselected;
+
 	/*
  -- ** -- P U B L I C   M E T H O D S -----------------------------------------
 	*/
@@ -125,7 +137,7 @@ public:
 
 	KeyframeTree();
 	~KeyframeTree();
-
+	//! Assign the model and connect signals from the canvas interface
 	void set_model(Glib::RefPtr<KeyframeTreeStore> keyframe_tree_store_);
 
 	void set_editable(bool x=true);
@@ -142,6 +154,13 @@ public:
 	sigc::signal<void,synfig::Keyframe,synfig::String>& signal_edited_description() { return signal_edited_description_; }
 
 	sigc::signal<void,int, Gtk::TreeRow, ColumnID>& signal_user_click() { return signal_user_click_; }
+
+	/*
+ -- ** -- P R O T E C T E D   M E T H O D S ---------------------------------------
+	*/
+
+	protected:
+
 }; // END of KeyframeTree
 
 }; // END of namespace studio
