@@ -71,6 +71,7 @@
 #include "workarearenderer/workarearenderer.h"
 #include "workarearenderer/renderer_background.h"
 #include "workarearenderer/renderer_canvas.h"
+#include "workarearenderer/renderer_frameerror.h"
 #include "workarearenderer/renderer_grid.h"
 #include "workarearenderer/renderer_guides.h"
 #include "workarearenderer/renderer_timecode.h"
@@ -193,6 +194,7 @@ WorkArea::WorkArea(etl::loose_handle<synfigapp::CanvasInterface> canvas_interfac
 	insert_renderer(new Renderer_Dragbox,    400);
 	insert_renderer(new Renderer_Timecode,   500);
 	insert_renderer(new Renderer_BoneSetup,  501);
+	insert_renderer(new Renderer_FrameError, 502);
 
 	signal_duck_selection_changed().connect(sigc::mem_fun(*this,&studio::WorkArea::queue_draw));
 	signal_duck_selection_single().connect(sigc::mem_fun(*this, &studio::WorkArea::on_duck_selection_single));
@@ -327,8 +329,8 @@ WorkArea::~WorkArea()
 	set_drag_mode(DRAG_NONE);
 	while(!renderer_set_.empty())
 		erase_renderer(*renderer_set_.begin());
-	if (getenv("SYNFIG_DEBUG_DESTRUCTORS"))
-		info("WorkArea::~WorkArea(): Deleted");
+	DEBUG_LOG("SYNFIG_DEBUG_DESTRUCTORS",
+		"WorkArea::~WorkArea(): Deleted");
 }
 
 void
@@ -1023,7 +1025,7 @@ WorkArea::on_drawing_area_event(GdkEvent *event)
 	SYNFIG_EXCEPTION_GUARD_BEGIN()
 	synfig::Point mouse_pos;
     float bezier_click_pos(0);
-	const float radius((abs(pw)+abs(ph))*4);
+	const float radius((std::fabs(pw)+std::fabs(ph))*4);
 	int button_pressed(0);
 	float pressure(0);
 	Gdk::ModifierType modifier(Gdk::ModifierType(0));
@@ -1031,7 +1033,7 @@ WorkArea::on_drawing_area_event(GdkEvent *event)
 	// Handle input stuff
 	if (event->any.type==GDK_MOTION_NOTIFY)
 	{
-		GdkDevice *device = event->motion.device;
+		GdkDevice *device = gdk_event_get_source_device(event);
 		modifier = Gdk::ModifierType(event->motion.state);
 
 		// Calculate the position of the
@@ -1096,7 +1098,7 @@ WorkArea::on_drawing_area_event(GdkEvent *event)
 		event->any.type==GDK_3BUTTON_PRESS ||
 		event->any.type==GDK_BUTTON_RELEASE )
 	{
-		GdkDevice *device = event->button.device;
+		GdkDevice *device = gdk_event_get_source_device(event);
 		modifier = Gdk::ModifierType(event->button.state);
 		drawing_area->grab_focus();
 
@@ -2102,6 +2104,20 @@ WorkArea::sync_render(bool refresh)
 	if (refresh) renderer_canvas->clear_render();
 	renderer_canvas->enqueue_render();
 	renderer_canvas->wait_render();
+
+	// Check for frame rendering error while playing the animation, report it and stops the playback at problematic frame
+	if (!canvas_view || !canvas_view->time_model())
+		return;
+	std::set<std::string> error_msg_set;
+	renderer_canvas->get_rendering_error_messages_for_time(canvas_view->time_model()->get_time(), error_msg_set);
+
+	if (!error_msg_set.empty()) {
+		std::string error_msg;
+		for (auto& msg : error_msg_set)
+			error_msg.append(msg + "\n");
+		App::get_ui_interface()->error(error_msg);
+		canvas_view->stop_async();
+	}
 }
 
 void
