@@ -74,7 +74,7 @@ Action::LayerEmbed::get_param_vocab()
 
 	ret.push_back(ParamDesc("layer",Param::TYPE_LAYER)
 		.set_local_name(_("Layer"))
-		.set_desc(_("Layer to be embed"))
+		.set_desc(_("Layer to be embedded"))
 	);
 
 	return ret;
@@ -92,7 +92,7 @@ Action::LayerEmbed::is_candidate(const ParamList &x)
 		= etl::handle<synfig::Layer_PasteCanvas>::cast_dynamic(layer);
 	if (layer_pastecanvas)
 	{
-		Canvas::Handle canvas = layer_pastecanvas->get_sub_canvas();;
+		Canvas::Handle canvas = layer_pastecanvas->get_sub_canvas();
 		if (canvas && canvas->is_root())
 			return true;
 	}
@@ -206,8 +206,6 @@ Action::LayerEmbed::prepare()
 
 		get_canvas_interface()->get_instance()->generate_new_name(
 			layer_import,
-			get_canvas(),
-			get_canvas()->get_file_system(),
 			new_description,
 			new_filename,
 			new_filename_param );
@@ -221,9 +219,11 @@ Action::LayerEmbed::prepare()
 			get_canvas_interface()->get_instance();
 		etl::handle<Layer_Bitmap> layer_bitmap =
 			etl::handle<Layer_Bitmap>::cast_dynamic(layer_import);
-		if (layer_bitmap && instance->is_layer_registered_to_save(layer_bitmap)) {
-			// save surface
-			get_canvas_interface()->get_instance()->save_surface(layer_bitmap->rendering_surface, new_filename);
+		if (layer_bitmap && layer_bitmap->is_surface_modified()) {
+			// save surface to new place
+			instance->save_surface(layer_bitmap->rendering_surface, new_filename);
+			//layer_bitmap->is_surface_modified();
+			layer_bitmap->reset_surface_modification_id();
 		} else {
 			// try to create directory
 			if (!file_system->directory_create_recursive(etl::dirname(new_filename)))
